const helpers = require('./helpers')
const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const collectionHelper = require('./Collection')

const setupNewUser = async (
  body,
  url,
  res,
  email,
  role,
  UserModel,
  invitationType,
) => {
  const { firstName, lastName, affiliation, title } = body
  const newUser = await helpers.createNewUser(
    email,
    firstName,
    lastName,
    affiliation,
    title,
    UserModel,
    role,
  )

  try {
    if (role !== 'reviewer') {
      await mailService.sendSimpleEmail({
        toEmail: newUser.email,
        user: newUser,
        emailType: invitationType,
        dashboardUrl: url,
      })
    }

    return newUser
  } catch (e) {
    logger.error(e.message)
    return { status: 500, error: 'Email could not be sent.' }
  }
}

const getEditorInChief = async UserModel => {
  const users = await UserModel.all()
  const eic = users.find(user => user.editorInChief || user.admin)
  return eic
}

const setupReviewerDecisionEmailData = async ({
  baseUrl,
  UserModel,
  FragmentModel,
  collection,
  user,
  mailService,
  agree,
  timestamp = Date.now(),
}) => {
  const {
    title,
    authorName,
    id,
  } = await collectionHelper.getFragmentAndAuthorData({
    UserModel,
    FragmentModel,
    collection,
  })
  const eic = await getEditorInChief(UserModel)
  const toEmail = collection.handlingEditor.email
  await mailService.sendNotificationEmail({
    toEmail,
    user,
    emailType: agree ? 'reviewer-agreed' : 'reviewer-declined',
    meta: {
      collection: { customId: collection.customId, id: collection.id },
      fragment: { id, title, authorName },
      handlingEditorName: collection.handlingEditor.name,
      baseUrl,
      eicName: `${eic.firstName} ${eic.lastName}`,
      timestamp,
    },
  })
  if (agree)
    await mailService.sendNotificationEmail({
      toEmail: user.email,
      user,
      emailType: 'reviewer-thank-you',
      meta: {
        collection: { customId: collection.customId, id: collection.id },
        fragment: { id, title, authorName },
        handlingEditorName: collection.handlingEditor.name,
        baseUrl,
      },
    })
}

const setupReviewerUnassignEmail = async ({
  UserModel,
  FragmentModel,
  collection,
  user,
  mailService,
}) => {
  const { title, authorName } = await collectionHelper.getFragmentAndAuthorData(
    {
      UserModel,
      FragmentModel,
      collection,
    },
  )

  await mailService.sendNotificationEmail({
    toEmail: user.email,
    user,
    emailType: 'unassign-reviewer',
    meta: {
      collection: { customId: collection.customId },
      fragment: { title, authorName },
      handlingEditorName: collection.handlingEditor.name,
    },
  })
}

module.exports = {
  setupNewUser,
  getEditorInChief,
  setupReviewerDecisionEmailData,
  setupReviewerUnassignEmail,
}
