const config = require('config')

const statuses = config.get('statuses')

const updateStatusByRecommendation = async (collection, recommendation) => {
  let newStatus = 'pendingApproval'
  if (['minor', 'major'].includes(recommendation))
    newStatus = 'revisionRequested'
  collection.status = newStatus
  collection.visibleStatus = statuses[collection.status].private
  await collection.save()
}

const updateFinalStatusByRecommendation = async (
  collection,
  recommendation,
) => {
  let newStatus
  switch (recommendation) {
    case 'reject':
      newStatus = 'rejected'
      break
    case 'publish':
      newStatus = 'published'
      break
    case 'return-to-handling-editor':
      newStatus = 'reviewCompleted'
      break
    default:
      break
  }
  collection.status = newStatus
  collection.visibleStatus = statuses[collection.status].private
  await collection.save()
}

const updateStatus = async (collection, newStatus) => {
  collection.status = newStatus
  collection.visibleStatus = statuses[collection.status].private
  await collection.save()
}

const getFragmentAndAuthorData = async ({
  UserModel,
  fragment,
  collection: { authors, handlingEditor },
}) => {
  const heRecommendation = fragment.recommendations.find(
    rec => rec.userId === handlingEditor.id,
  )
  let { title, abstract } = fragment.metadata
  const { type } = fragment.metadata
  title = title.replace(/<(.|\n)*?>/g, '')
  abstract = abstract ? abstract.replace(/<(.|\n)*?>/g, '') : ''

  const submittingAuthorData = authors.find(
    author => author.isSubmitting === true,
  )
  const submittingAuthor = await UserModel.find(submittingAuthorData.userId)
  const authorsPromises = authors.map(async author => {
    const user = await UserModel.find(author.userId)
    return `${user.firstName} ${user.lastName}`
  })
  const authorsList = await Promise.all(authorsPromises)
  return {
    title,
    submittingAuthor,
    abstract,
    authorsList,
    type,
    heRecommendation,
  }
}

const getAgreedReviewerInvitation = (invitations = []) =>
  invitations.filter(
    inv =>
      inv.role === 'reviewer' &&
      inv.hasAnswer === true &&
      inv.isAccepted === true,
  )

module.exports = {
  updateStatusByRecommendation,
  getFragmentAndAuthorData,
  getAgreedReviewerInvitation,
  updateStatus,
  updateFinalStatusByRecommendation,
}
