const helpers = require('../../helpers/helpers')
const uuid = require('uuid')
const authsomeHelper = require('../../helpers/authsome')
const collectionHelper = require('../../helpers/Collection')
const userHelper = require('../../helpers/User')
const get = require('lodash/get')

module.exports = models => async (req, res) => {
  const { recommendation, comments, recommendationType } = req.body
  if (!helpers.checkForUndefinedParams(recommendationType))
    return res.status(400).json({ error: 'Recommendation type is required.' })

  const reqUser = await models.User.find(req.user)
  const { collectionId, fragmentId } = req.params

  let collection, fragment

  try {
    collection = await models.Collection.find(collectionId)
    if (!collection.fragments.includes(fragmentId))
      return res.status(400).json({
        error: `Collection and fragment do not match.`,
      })

    fragment = await models.Fragment.find(fragmentId)
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'Item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
  const authsome = authsomeHelper.getAuthsome(models)
  const target = {
    collection,
    path: req.route.path,
  }
  const canPost = await authsome.can(req.user, 'POST', target)
  if (!canPost)
    return res.status(403).json({
      error: 'Unauthorized.',
    })
  fragment.recommendations = fragment.recommendations || []
  const newRecommendation = {
    id: uuid.v4(),
    userId: reqUser.id,
    createdOn: Date.now(),
    updatedOn: Date.now(),
    recommendationType,
  }

  newRecommendation.recommendation = recommendation || undefined
  newRecommendation.comments = comments || undefined
  const UserModel = models.User

  const {
    title,
    submittingAuthor,
    type,
    heRecommendation,
  } = await collectionHelper.getFragmentAndAuthorData({
    UserModel,
    fragment,
    collection,
  })
  const baseUrl = helpers.getBaseUrl(req)
  const authorName = `${submittingAuthor.firstName} ${
    submittingAuthor.lastName
  }`
  if (reqUser.editorInChief || reqUser.admin) {
    if (recommendation === 'return-to-handling-editor')
      await collectionHelper.updateStatus(collection, 'reviewCompleted')
    else {
      await collectionHelper.updateFinalStatusByRecommendation(
        collection,
        recommendation,
      )
      await userHelper.setupAuthorsEmailData({
        baseUrl,
        UserModel,
        collection,
        fragment: { title, submittingAuthor },
        comments: get(heRecommendation, 'comments'),
        requestToRevision: false,
        publish: recommendation === 'publish',
      })
      await userHelper.setupManuscriptDecisionEmailForHe({
        UserModel,
        fragment: { title, submittingAuthor },
        collection: {
          customId: collection.customId,
          handlingEditor: collection.handlingEditor,
        },
        publish: recommendation === 'publish',
      })

      await userHelper.setupReviewersEmail({
        UserModel,
        collection,
        fragment: {
          title,
          authorName,
          recommendations: fragment.recommendations,
        },
        recommendation,
        isSubmitted: true,
      })
    }
  } else if (recommendationType === 'editorRecommendation') {
    await collectionHelper.updateStatusByRecommendation(
      collection,
      recommendation,
    )
    await userHelper.setupReviewersEmail({
      UserModel,
      collection,
      fragment: {
        title,
        authorName,
        recommendations: fragment.recommendations,
      },
      recommendation,
    })
    await userHelper.setupNoResponseReviewersEmailData({
      baseUrl,
      UserModel,
      collection,
      fragment: { title, authorName, type },
    })
    await userHelper.setupEiCRecommendationEmailData({
      baseUrl,
      UserModel,
      collection,
      recommendation,
      fragment: { title, id: fragment.id, authorName },
      comments: newRecommendation.comments,
    })
    if (['minor', 'major'].includes(recommendation))
      await userHelper.setupAuthorsEmailData({
        baseUrl,
        UserModel,
        collection,
        fragment: { title, id: fragment.id, submittingAuthor },
        comments: newRecommendation.comments,
        requestToRevision: true,
      })
  }

  fragment.recommendations.push(newRecommendation)
  await fragment.save()
  return res.status(200).json(newRecommendation)
}
